Date: 2022-10-01T06:40:16+00:00

Actual source code: test11.c
slepc-3.18.0 2022-10-01
Report Typos and Errors

      1: /*
      2:    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      3:    SLEPc - Scalable Library for Eigenvalue Problem Computations
      4:    Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain

      6:    This file is part of SLEPc.
      7:    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
      8:    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      9: */

     11: static char help[] = "Tests a user-defined convergence test (based on ex8.c).\n\n"
     12:   "The command line options are:\n"
     13:   "  -n <n>, where <n> = matrix dimension.\n\n";

     15: #include <slepcsvd.h>

     17: /*
     18:    This example computes the singular values of an nxn Grcar matrix,
     19:    which is a nonsymmetric Toeplitz matrix:

     21:               |  1  1  1  1               |
     22:               | -1  1  1  1  1            |
     23:               |    -1  1  1  1  1         |
     24:               |       .  .  .  .  .       |
     25:           A = |          .  .  .  .  .    |
     26:               |            -1  1  1  1  1 |
     27:               |               -1  1  1  1 |
     28:               |                  -1  1  1 |
     29:               |                     -1  1 |

     31:  */

     33: /*
     34:   MyConvergedRel - Convergence test relative to the norm of A (given in ctx).
     35: */
     36: PetscErrorCode MyConvergedRel(SVD svd,PetscReal sigma,PetscReal res,PetscReal *errest,void *ctx)
     37: {
     38:   PetscReal norm = *(PetscReal*)ctx;

     40:   *errest = res/norm;
     41:   return 0;
     42: }

     44: int main(int argc,char **argv)
     45: {
     46:   Mat            A;               /* Grcar matrix */
     47:   SVD            svd;             /* singular value solver context */
     48:   PetscInt       N=30,Istart,Iend,i,col[5],nconv1,nconv2;
     49:   PetscScalar    value[] = { -1, 1, 1, 1, 1 };
     50:   PetscReal      sigma_1,sigma_n;

     53:   SlepcInitialize(&argc,&argv,(char*)0,help);

     55:   PetscOptionsGetInt(NULL,NULL,"-n",&N,NULL);
     56:   PetscPrintf(PETSC_COMM_WORLD,"\nEstimate the condition number of a Grcar matrix, n=%" PetscInt_FMT "\n\n",N);

     58:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     59:         Generate the matrix
     60:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

     62:   MatCreate(PETSC_COMM_WORLD,&A);
     63:   MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,N,N);
     64:   MatSetFromOptions(A);
     65:   MatSetUp(A);
     66:   MatGetOwnershipRange(A,&Istart,&Iend);
     67:   for (i=Istart;i<Iend;i++) {
     68:     col[0]=i-1; col[1]=i; col[2]=i+1; col[3]=i+2; col[4]=i+3;
     69:     if (i==0) MatSetValues(A,1,&i,PetscMin(4,N-i),col+1,value+1,INSERT_VALUES);
     70:     else MatSetValues(A,1,&i,PetscMin(5,N-i+1),col,value,INSERT_VALUES);
     71:   }
     72:   MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);
     73:   MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);

     75:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     76:              Create the SVD solver and set the solution method
     77:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

     79:   SVDCreate(PETSC_COMM_WORLD,&svd);
     80:   SVDSetOperators(svd,A,NULL);
     81:   SVDSetType(svd,SVDTRLANCZOS);
     82:   SVDSetFromOptions(svd);

     84:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     85:                       Solve the singular value problem
     86:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

     88:   SVDSetWhichSingularTriplets(svd,SVD_LARGEST);
     89:   SVDSolve(svd);
     90:   SVDGetConverged(svd,&nconv1);
     91:   if (nconv1 > 0) SVDGetSingularTriplet(svd,0,&sigma_1,NULL,NULL);
     92:   else PetscPrintf(PETSC_COMM_WORLD," Unable to compute large singular value!\n\n");

     94:   /* compute smallest singular value relative to the matrix norm */
     95:   SVDSetConvergenceTestFunction(svd,MyConvergedRel,&sigma_1,NULL);
     96:   SVDSetWhichSingularTriplets(svd,SVD_SMALLEST);
     97:   SVDSolve(svd);
     98:   SVDGetConverged(svd,&nconv2);
     99:   if (nconv2 > 0) SVDGetSingularTriplet(svd,0,&sigma_n,NULL,NULL);
    100:   else PetscPrintf(PETSC_COMM_WORLD," Unable to compute small singular value!\n\n");

    102:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    103:                     Display solution and clean up
    104:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
    105:   if (nconv1 > 0 && nconv2 > 0) {
    106:     PetscPrintf(PETSC_COMM_WORLD," Computed singular values: sigma_1=%.4f, sigma_n=%.4f\n",(double)sigma_1,(double)sigma_n);
    107:     PetscPrintf(PETSC_COMM_WORLD," Estimated condition number: sigma_1/sigma_n=%.4f\n\n",(double)(sigma_1/sigma_n));
    108:   }

    110:   SVDDestroy(&svd);
    111:   MatDestroy(&A);
    112:   SlepcFinalize();
    113:   return 0;
    114: }

    116: /*TEST

    118:    test:
    119:       suffix: 1

    121: TEST*/